#include <stdlib.h>
#include <timer.h>
#include <display.h>
#include <keyboard.h>

#include "../include/load.h"

#define FPS (int)(1000 / 50)

/*
 * Private declarations
 */

int timestamp;
tracksheet_t *track = NULL;
note_t *notes = NULL;

int debug_note = 0;

void editor_handler(void);
void editor_draw(void);


/*
 * Public functions
 */

void editor(char *filename)
{
	int run = 1;
	int row = -1;

	track = malloc(sizeof(tracksheet_t));

	track->title = "Track title";
	track->difficulty = HARD;
	track->length = 0;
	track->n_notes = 0;
	track->notes = notes;

	timestamp = 0;

	timer_t *timer = timer_create(FPS, 0xfffffff);
	timer_attach(timer, editor_handler, NULL);
	timer_start(timer);

	while(run)
	{
		switch (getkey()) {
			case KEY_F1:
				row = 0;
				break;
			case KEY_F2:
				row = 1;
				break;
			case KEY_F3:
				row = 2;
				break;
			case KEY_F4:
				row = 3;
				break;
			case KEY_F5:
				row = 4;
				break;
			case KEY_F6:
				row = 5;
				break;
			case KEY_EXIT:
				run = 0;
			default:
				row = -1;
		}

		if(row >= 0)
		{
			if(track->n_notes == 0)
			{
				notes = malloc(sizeof(note_t));
				track->notes = notes;
			}
			else
			{
				notes = realloc(notes, (track->n_notes + 1) * sizeof(note_t));
				track->notes = notes;
			}

			notes[track->n_notes].row = row;
			notes[track->n_notes].time = timestamp;

			debug_note = track->n_notes;

			track->n_notes++;
		}
	}

	timer_stop(timer);
	timer_destroy(timer);

	track->length = timestamp;

	save_track(track, filename);

	free_track(track);
	free(notes);
}

/*
 * Private functions
 */

void editor_handler(void)
{
	timestamp++;

	editor_draw();
}

void editor_draw(void)
{
	int i = 0;

	extern image_t gbackground;
	extern image_t balls;

	dclear();

	dimage(0, 0, &gbackground);

	while(i < track->n_notes)
	{
		if(timestamp - track->notes[i].time < 70)
		{
			dimage_part(11 + track->notes[i].row * 21 - 3,
				1 * (track->notes[i].time - timestamp) + 53 - 3,
				&balls, 0, 0, 7, 7);
		}
		i++;
	}

	dupdate();
}
