//------------------------------
// TODO:
// * Better animations handling
//------------------------------

#define FPS (int)(1000 / 50)

#include <stdlib.h>
#include <timer.h>
#include <display.h>
#include <keyboard.h>

#include "../include/load.h"


/*
 * Private declarations
 */

struct image_part {
	int x, y;
	int width, height;
};

struct explosion_s {
	int row;
	int frame;
};

void dexplosion(void);
void add_explosion(int row);

void draw(void);
void engine_handler(void);
void hit(int row);

int timestamp;
int run;
int score, combo;
char *msg = NULL;
tracksheet_t *track;
int current_note;
struct explosion_s explosions[6];

/*
 * Public functions
 */

void engine(char *tracksheet)
{
	int row;

	track = load_track(tracksheet);
	timestamp = 0;
	score = 0;
	current_note = 0;
	msg = "";
	run = 1;

	dclear();
	dprint(0, 0, "Laddr: %p", track);
	dupdate();
	getkey();

	for(int i = 0; i < 6; i++)
	{
 		explosions[i].row = i;
		explosions[i].frame = -1;
	}

	timer_t *timer = timer_create(FPS, 0xfffffff);
	timer_attach(timer, engine_handler, NULL);
	timer_start(timer);

	while(run)
	{
		switch (getkey()) {
			case KEY_F1:
				row = 0;
				break;
			case KEY_F2:
				row = 1;
				break;
			case KEY_F3:
				row = 2;
				break;
			case KEY_F4:
				row = 3;
				break;
			case KEY_F5:
				row = 4;
				break;
			case KEY_F6:
				row = 5;
				break;
			case KEY_EXIT:
				run = 0;
			default:
				row = -1;
		}

		if(row >= 0)
		{
			hit(row);
			add_explosion(row);
		}
	}

	timer_stop(timer);
	timer_destroy(timer);

	dclear();
	dprint(0, 0, "Fin de partie");
	dupdate();
	getkey();

	dclear();
	dprint(0, 0, "Faddr: %p", track);
	dupdate();
	getkey();

	free_track(track);
	track = NULL;

	dprint(0,8, "Freed track!");
	dupdate();
	getkey();
}


/*
 * Private functions
 */


void engine_handler(void)
{
	int i;

	timestamp++;

	if(timestamp > track->length)
		run = 0;

	draw();
}

void draw(void)
{
	extern image_t gbackground;
	extern image_t balls;
	int i = current_note;

	dclear();

	dimage(0, 0, &gbackground);

	while(track->notes[i].time < timestamp + 70)
	{
		if(track->notes[i].row >= 0)
		{
			dimage_part(11 + track->notes[i].row * 21 - 3,
				1 * (timestamp - track->notes[i].time) + 53 - 3,
				&balls, 0, 0, 7, 7);
		}
		i++;
	}

	dexplosion();
	dprint(0, 0, "%d", score);
	dprint(64, 0, msg);

	dupdate();
}

void dexplosion(void)
{
	extern image_t explosion;
	struct image_part frames[] = {
		{0, 8, 5, 5},
		{5, 6, 9, 9},
		{14, 4, 13, 13},
		{27, 2, 17, 17},
		{44, 0, 21, 21}
	};

	for(int i = 0; i < 6; i++)
	{
		if(explosions[i].frame > -1)
		{
			int frame = explosions[i].frame / 2;

			dimage_part(11 + i * 21 - frames[frame].width / 2,
				53 - frames[frame].height / 2,
				&explosion, frames[frame].x, frames[frame].y,
				frames[frame].width, frames[frame].height);

			explosions[i].frame++;

			if(explosions[i].frame >= 10)
			{
				explosions[i].frame = -1;
			}
		}
	}
}

void hit(int row)
{
	int i = current_note;
	int delay;

	while(track->notes[i].row != row) i++;

	delay = track->notes[i].time - timestamp;

	if(delay > 50)
	{
		combo = 0;
		return;
	}

	switch ((int)(abs(delay) / track->difficulty)) {
		case 0:
			score += 3 * (1 + combo);
			combo++;
			msg = "Perfect!";
			break;
		case 1:
		case 2:
			score += 2 * (1 + combo);
			combo++;
			msg = "Good";
			break;
		case 3:
		case 4:
		case 5:
			score += 1 * (1 + combo);
			combo++;
			msg = "Poor";
			break;
		default:
			combo = 0;
			msg = "Miss...";
	}

	track->notes[i].row = -1;

	i = current_note;

	while(track->notes[i].time < timestamp - 30) i++;
	current_note = i;
}

void add_explosion(int row)
{
	explosions[row].frame = 0;
}
