#include <bfile.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#include "../include/load.h"
#include "../include/tracksheet.h"

#include <display.h>
#include <keyboard.h>

#define debug_stop(s) dclear();dtext(0, 0, s);dupdate();getkey()

/*
 * Private functions declarations
 */

int file_open(const char *filename, enum BFile_OpenMode mode);
int file_create(const char *filename, int size);
int len_int(int n);
int str2int(const char *str);

// gint quick fix
int BFile_Size(int handle);


/*
 * Public functions
 */

tracksheet_t *load_track(const char *filename)
{
	char *buffer = NULL;
	char *_buffer = NULL;
	int i;
	int size;
	char *ptr = NULL;

	int handle = file_open(filename, BFile_ReadOnly);
	tracksheet_t *track = NULL;
	note_t *notes = NULL;

	/*
	 * reading file size
	 */

	size = BFile_Size(handle);

	/*
	 * reading header
	 */

	track = (tracksheet_t*) malloc(sizeof(tracksheet_t));
	buffer = (char*) malloc(size);
	_buffer = buffer;

	BFile_Read(handle, _buffer, size, 0);
	BFile_Close(handle);

	// title
	ptr = strchr(_buffer, '\n');
	track->title = malloc((int)(ptr - _buffer + 1));
	memcpy(track->title, _buffer, (int)(ptr - _buffer));
	track->title[ptr - _buffer] = '\0';
	_buffer = ptr + 1;

	// difficulty
	ptr = strchr(_buffer, ';');
	*ptr = '\0';
	track->difficulty = str2int(_buffer);
	_buffer = ptr + 1;

	// length
	ptr = strchr(_buffer, ';');
	*ptr = '\0';
	track->length = str2int(_buffer);
	_buffer = ptr + 1;

	// notes
	ptr = strchr(_buffer, '\n');
	*ptr = '\0';
	track->n_notes = str2int(_buffer);
	_buffer = ptr + 1;

	/*
	 * parsing notes
	 */

	notes = malloc(track->n_notes * sizeof(note_t));
	track->notes = notes;

	dclear();
	dprint(0, 0, "Title: %s", track->title);
	dprint(0, 8, "Diff: %d", track->difficulty);
	dprint(0, 16, "Length: %d", track->length);
	dprint(0, 24, "n_notes: %d", track->n_notes);
	dupdate();
	getkey();

	i = 0;
	while(strchr(_buffer, '\n') != NULL)
	{
		ptr = strchr(_buffer, ',');

		*ptr = '\0';

		notes[i].row = (char) str2int(_buffer);
		_buffer = ptr + 1;

		ptr = strchr(_buffer, '\n');
		*ptr = '\0';

		notes[i].time = (short) str2int(_buffer);

		i++;
		if(i == track->n_notes)
		{
			break;
		}
		else
		{
			_buffer = ptr + 1;
		}
	}

	free(buffer);

	debug_stop("Load finished!");

	return track;
}

void save_track(tracksheet_t* track, const char *filename)
{
	int size = strlen(track->title) + 1 // title
		+ len_int(track->difficulty) + 1 // difficulty
		+ len_int(track->length) + 1 // length
		+ len_int(track->n_notes) + 1; // n_notes

	for(int i = 0; i < track->n_notes; i++)
	{
		size += len_int(track->notes[i].row) + 1
			+ len_int(track->notes[i].time) + 1;
	}

	// complete to 4 bytes
	size--;
	size += 4 - (size % 4);

	char *buffer = malloc(size);
	char *ptr = buffer;

	// title
	ptr += sprintf(ptr, "%s\n", track->title);
	ptr += sprintf(ptr, "%d;", track->difficulty);
	ptr += sprintf(ptr, "%d;", track->length);
	ptr += sprintf(ptr, "%d\n", track->n_notes);

	for(int i = 0; i < track->n_notes; i++)
	{
		ptr += sprintf(ptr, "%d,%d\n", track->notes[i].row, track->notes[i].time);
	}

	*ptr = '\0';

	file_create(filename, size);
	int handle = file_open(filename, BFile_WriteOnly);
	BFile_Write(handle, buffer, size);
	BFile_Close(handle);

	int x = 0;
	int y = 0;

	free(buffer);

	dclear();
	dprint(0, 0, "Save completed!");
	dupdate();
	getkey();
}

void free_track(tracksheet_t *track)
{
	free(track->title);
	free(track->notes);
	free(track);
}


/*
 * Private functions
 */

int file_open(const char *filename, enum BFile_OpenMode mode)
{
	uint16_t *casioname = NULL;
	casioname = (uint16_t*) malloc(sizeof(uint16_t) * (strlen(filename) + 1));

	for(unsigned int i = 0; i < strlen(filename); i++)
	{
		casioname[i] = filename[i];
	}
	casioname[strlen(filename)] = '\0';

	int handle = BFile_Open(casioname, mode);
	free(casioname);

	return handle;
}

int file_create(const char *filename, int size)
{
	uint16_t *casioname = NULL;
	casioname = (uint16_t*) malloc(sizeof(uint16_t) * (strlen(filename) + 1));

	for(unsigned int i = 0; i < strlen(filename); i++)
	{
		casioname[i] = filename[i];
	}
	casioname[strlen(filename)] = '\0';

	BFile_Remove(casioname);
	BFile_Create(casioname, BFile_File, &size);

	free(casioname);
}

int len_int(int n)
{
	if(n == 0)
		return 1;

	int i = 0;

	while(n != 0)
	{
		n /= 10;
		i++;
	}

	return i;
}

int str2int(const char *str)
{
	int i = 0;
	int n = 0;

	while(str[i] != '\0')
	{
		n *= 10;
		n += str[i] - '0';
		i++;
	}

	return n;
}
